package cn.edu.zjvtit.cloudstorage.dataservice.controller;

import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;
import cn.edu.zjvtit.cloudstorage.dataservice.result.Result;

import javax.servlet.http.HttpServletRequest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

@RestController
public class TempController {
    @Autowired
    Logger logger;

    @PostMapping("/temp")
    public String postTemp(@RequestHeader Map<String, String> requestHeader) {
        String filename = UUID.randomUUID().toString();
        String size = requestHeader.get("Size");
        //todo check disk free space
        Result ret = new Result("success", "10000");
        Map data = new HashMap<String, String>();
        data.put("uuid", filename);
        ret.setData(data);
        return ret.toString();
    }

    @PatchMapping("/temp/{uuid}")
    public String patchTemp(@PathVariable("uuid") String uuid,
                            HttpServletRequest request) throws IOException {
        // 存储文件
        logger.info("received put object request");
        File fileToSave = new File("./temp/" + uuid);
        InputStream is = request.getInputStream();
        FileOutputStream os = new FileOutputStream(fileToSave);
        byte[] buff = new byte[1024];
        int i = 0;
        while ((i = is.read(buff)) != -1) {
            os.write(buff, 0, i);
            os.flush();
        }
        os.close();
        String md5 = DigestUtils.md5DigestAsHex(is);
        return (new Result("success", "10000").withData((Map<String, Object>) new HashMap<>().put("md5", md5))).toString();
    }

    @PutMapping("/temp/{uuid}")
    public String putTemp(@PathVariable("uuid") String uuid,
                          @RequestHeader Map<String, String> requestHeader) throws IOException {
        String md5 = requestHeader.get("md5");
        File fileToMove = new File("./temp/" + uuid);
        File fileToSave = new File("./objects/" + md5);
        if (fileToMove.renameTo(fileToSave)) {
            return (new Result("success", "10000")).toString();
        } else {
            return (new Result("fail", "10002")).toString();
        }
    }

    @DeleteMapping("/temp/{uuid}")
    public String delTemp(@PathVariable("uuid") String uuid) throws IOException {
        File fileToDelete = new File("./temp/" + uuid);
        if (fileToDelete.delete()) {
            return (new Result("success", "10000")).toString();
        } else {
            return (new Result("fail", "10002")).toString();
        }
    }

}
